/**
  * Bouton plat et transparent, developpé initialement pour le projet
  * Imageo (le-terrier d'AbulEdu) et repris dans les classes de base
  * d'AbulÉdu (merci Thomas !)
  *
  * Utilisation: https://redmine.ryxeo.com/projects/ryxeo/wiki/AbulEduFlatBoutonV1
  *
  * #AbulEduFlatBoutonV1
  * include(lib/abuleduflatboutonv1/abuleduflatboutonv1.pri)
  *
  * @see https://redmine.ryxeo.com/projects/leterrier-imageo
  * @author 2011 Thomas Grosjean-Pasut <thomas.grosjeanpasut@ryxeo.com>
  * @author 2011 Eric Seigne <eric.seigne@ryxeo.com>
  * @author 2011 Jean-Louis Frucot <frucot.jeanlouis@free.fr>
  * @see The GNU Public License (GPL)
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef ABULEDUFLATBOUTONV1_H
#define ABULEDUFLATBOUTONV1_H

#include <QPushButton>
#include <QGraphicsSceneMouseEvent>
#include <QFile>
#include <QTimer>
#include <QApplication>
#include <QtCore/qmath.h>

class AbulEduFlatBoutonV1 : public QPushButton
{
    Q_OBJECT

public:
    AbulEduFlatBoutonV1(QWidget* parent = 0);
    ~AbulEduFlatBoutonV1();

    void setCouleurTexteNormale(QColor couleur);
    void setCouleurTexteSurvol(QColor couleur);
    void setCouleurTextePressed(QColor couleur);
    void setCouleurTexteDisabled(QColor couleur);
    void setCouleursTexte(QColor normal, QColor survol = QColor(QColor::Invalid), QColor pressed = QColor(QColor::Invalid), QColor disabled = QColor(QColor::Invalid));

    void setTexteAlignement(Qt::Alignment alignement = Qt::AlignHCenter);
    void setTextePaddingLeft(qreal padding);

    void setCouleurFondNormale(QColor couleur);
    void setCouleurFondSurvol(QColor couleur);
    void setCouleurFondPressed(QColor couleur);
    void setCouleurFondDisabled(QColor couleur);
    void setCouleursFond(QColor normal, QColor survol = QColor(QColor::Invalid), QColor pressed = QColor(QColor::Invalid), QColor disabled = QColor(QColor::Invalid));

    void setIconeNormale(QString normale);
    void setIconeSurvol(QString survol);
    void setIconePressed(QString pressed);
    void setIconeDisabled(QString disabled);
    void setIcones(QString normale, QString survol = "", QString pressed = "", QString disabled = "" );

    /** Permet de passer une feuille de style au AbulEduFlatBouton
      * @warning Attention, tout appel à une autre fonction détruira cette feuille de style !
      * @warning La feuille de style passée doit être valide : il n'y a pas de parsing préalable !
      */
    void setCSSFlatBouton(QString css);
    void setDisableTimeout(int msec);


private:
    /** Map contenant les couleurs sous forme de QString rgba
      * prêtes à inclure dans une feuille de style : "(int r, int g, int b, int a)"
      */
    QMap<QString , QString> *m_couleursCSS;

    /** QMap contenant les chemins vers les fichiers d'image-icone */
    QMap<QString, QString> *m_icones;

    /** Contient la feuille de style des AbulEduFlatBoutonV1 */
    QString m_cssFlatBouton;

    /** Contient le déplacement horizontal (padding) */
    int m_paddingLeft;

    QString m_texteAlignement;
    bool m_changeIcon;
    /** Convertit une QColor en QString "(int r, int g, int b, int a)" */
    QString toRGBA(QColor couleur);

    /** Reconstruit la CSS du FlatBoutonV1 */
    void updateCSS();

    QTimer *m_timer;
    void mouseReleaseEvent(QMouseEvent *e);

    bool m_localDebug;

public slots:
    void slotDisableTimeout();
    void slotClicked();
signals:


};

#endif // ABULEDUFLATBOUTONV1_H
