/** Classe AbulEduDownloadProgressBar : popup de téléchargement
  *
  * @author 2012 Eric Seigne <eric.seigne@ryxeo.com>
  * @see The GNU Public License (GNU/GPL) v3
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#include "abuledudownloadprogressbar.h"
#include "ui_abuledudownloadprogressbar.h"

AbulEduDownloadProgressBar::AbulEduDownloadProgressBar(QNetworkReply *reply, QWidget *parent) :
    QWidget(parent),
    ui(new Ui::AbulEduDownloadProgressBar)
{
    ui->setupUi(this);
    connect(reply,SIGNAL(downloadProgress(qint64,qint64)),this,SLOT(slotLogicielDownloadInProgress(qint64,qint64)));
    m_time.start();
}

AbulEduDownloadProgressBar::~AbulEduDownloadProgressBar()
{
    delete ui;
}

void AbulEduDownloadProgressBar::abeUpdateStatus(QString message, int pourcentage)
{
    ui->progressBar->setValue(pourcentage);
    ui->lblTexte->setText(message);
}

void AbulEduDownloadProgressBar::slotLogicielDownloadInProgress(qint64 a, qint64 b)
{
    int actuel = a/1024/1024;
    int total = b/1024/1024; //En Mo
    int vitesse = a/m_time.elapsed(); //En Ko/sec
//    qDebug() << "Temps passé : " << m_time.elapsed() << " actuel : " << actuel << " total " << total << " vitesse " << vitesse;
    QString message = trUtf8("Temps restant : %1 minutes -- %2 sur %3 Mo (%4 Ko/s)").arg(QString::number(5)).arg(QString::number(actuel)).arg(QString::number(total)).arg(QString::number(vitesse));
    int pourcentage = 0;
    if(a > 0)
        pourcentage = (a*100/b);

//    qDebug() << "MainWindow::slotLogicielDownloadInProgress(1) : " << a << " et " << b << " soit " << pourcentage;
    abeUpdateStatus(message, pourcentage);
}

QByteArray AbulEduDownloadProgressBar::fileNameFromContentDisposition(const QByteArray &header)
{
    foreach(QByteArray item, header.split(';') )
    {
        int equalSignPos = item.indexOf('=');
        if (equalSignPos == -1)
            continue;
        QString key = item.left(equalSignPos).trimmed();
        if(key.compare("filename", Qt::CaseInsensitive) != 0)
            continue;

        QByteArray value = item.mid(equalSignPos+1).trimmed();
        if(value.startsWith('\"') && value.endsWith('\"'))
        {
            value = value.mid(1,value.length()-2);
        }
        return QByteArray::fromPercentEncoding(value);
    }
    return QByteArray();
}
