/** Classe AbulEduLocalSocketIpcClientV1 pour passage de ticket SSO entre applications
  * @author 2012 Eric Seigne <eric.seigne@ryxeo.com>
  * @see The GNU Public License (GNU/GPL) v3
  *
  * A partir de l'exemple suivant:
  * http://www.developer.nokia.com/Community/Wiki/Qt_IPC_using_QLocalSocket_&_QLocalServer
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#include "abuledulocalsocketipcclientv1.h"

#include <QtNetwork>
#include <QDebug>

AbulEduLocalSocketIpcClientV1::AbulEduLocalSocketIpcClientV1(QString remoteServername, QObject *parent) :
    QObject(parent) {
    if(m_localDebug)
        qDebug() << "AbulEduLocalSocketIpcClientV1::AbulEduLocalSocketIpcClientV1 (1)";

    m_socket = new QLocalSocket(this);
    m_serverName = remoteServername;

    m_useCrypt = true;

    connect(m_socket, SIGNAL(connected()), this, SLOT(abeSlotSocketConnected()));
    connect(m_socket, SIGNAL(disconnected()), this, SLOT(abeSlotSocketDisconnected()));

    connect(m_socket, SIGNAL(readyRead()), this, SLOT(abeSlotSocketReadReady()));
    connect(m_socket, SIGNAL(error(QLocalSocket::LocalSocketError)),
            this, SLOT(abeSlotSocketError(QLocalSocket::LocalSocketError)));
    if(m_localDebug)
        qDebug() << "AbulEduLocalSocketIpcClientV1::AbulEduLocalSocketIpcClientV1 (2)";
}

AbulEduLocalSocketIpcClientV1::~AbulEduLocalSocketIpcClientV1() {
    m_socket->abort();
    delete m_socket;
    m_socket = NULL;
}

void AbulEduLocalSocketIpcClientV1::abeSlotSendMessageToServer(QString message) {
    if(m_localDebug)
        qDebug() << "AbulEduLocalSocketIpcClientV1::abeSlotSendMessageToServer : " << message;

    m_socket->abort();
    m_message = message;
    m_socket->connectToServer(m_serverName);
}


void AbulEduLocalSocketIpcClientV1::abeSlotSocketConnected(){
    if(m_localDebug)
        qDebug() << "AbulEduLocalSocketIpcClientV1::abeSlotSocketConnected (1)";
    QByteArray block;
    QDataStream out(&block, QIODevice::WriteOnly);
    out.setVersion(QDataStream::Qt_4_0);
    //Quand alacarte se comporte comme un client il faut crypter
    if(qApp->applicationName() == "abuledu-alacarte" && m_useCrypt) {
        //C'est là qu'il faut gérer le trousseau de cles des logiciels ... et que ça va
        //devenir un peu compliqué
        SimpleCrypt crypted_message(Q_UINT64_C(0x0c011bfab2fba34e56faeb8a5e4f1ae23e2f0eb4));
        out << crypted_message.encryptToString(m_message);
    }
    else {
        out << m_message;
    }
    out.device()->seek(0);
    m_socket->write(block);
    m_socket->flush();
    if(m_localDebug)
        qDebug() << "AbulEduLocalSocketIpcClientV1::abeSlotSocketConnected (2)";
}

void AbulEduLocalSocketIpcClientV1::abeSlotSocketDisconnected() {
    if(m_localDebug)
        qDebug() << "AbulEduLocalSocketIpcClientV1::abeSlotSocketDisconnected: socket_disconnected";
}

void AbulEduLocalSocketIpcClientV1::abeSlotChangeRemoteServer(QString remoteServername)
{
    m_serverName = remoteServername;
}


void AbulEduLocalSocketIpcClientV1::abeSlotSocketReadReady() {
    if(m_localDebug)
        qDebug() << "AbulEduLocalSocketIpcClientV1::abeSlotSocketReadReady";
}

void AbulEduLocalSocketIpcClientV1::abeSlotSocketError(QLocalSocket::LocalSocketError) {
    if(m_localDebug)
        qDebug() << "AbulEduLocalSocketIpcClientV1::abeSlotSocketError";
    qCritical() << "AbulEduLocalSocketIpcClientV1::abeSlotSocketError";
}

