/** Classe AbulEduLocalSocketIpcServerV1 pour passage de ticket SSO entre applications
  * @author 2012 Eric Seigne <eric.seigne@ryxeo.com>
  * @see The GNU Public License (GNU/GPL) v3
  *
  * A partir de l'exemple suivant:
  * http://www.developer.nokia.com/Community/Wiki/Qt_IPC_using_QLocalSocket_&_QLocalServer
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#include "abuledulocalsocketipcserverv1.h"

#include <QtNetwork>
#include <QDebug>

AbulEduLocalSocketIpcServerV1::AbulEduLocalSocketIpcServerV1(QString servername, QObject *parent)
    :QObject(parent)
{
    m_localDebug = false;
    if(m_localDebug)
        qDebug() << "AbulEduLocalSocketIpcServerV1::AbulEduLocalSocketIpcServerV1 (1)";

    m_useCrypt = true;
    m_server = new QLocalServer(this);
    if (!m_server->listen(servername)) {
        if(m_server->isListening()) {
            if(m_localDebug)
                qDebug() << "AbulEduLocalSocketIpcServerV1::AbulEduLocalSocketIpcServerV1 : Server is listening";
        }
        else {
            if(m_localDebug)
                qDebug() << "AbulEduLocalSocketIpcServerV1::AbulEduLocalSocketIpcServerV1 : Not able to start the Server";
            qCritical() << "AbulEduLocalSocketIpcServerV1::AbulEduLocalSocketIpcServerV1 : Not able to start the Server";
        }
    }
    connect(m_server, SIGNAL(newConnection()), this, SLOT(abeSlotSocketNewConnection()));
    if(m_localDebug)
        qDebug() << "AbulEduLocalSocketIpcServerV1::AbulEduLocalSocketIpcServerV1 (2)";
}

AbulEduLocalSocketIpcServerV1::~AbulEduLocalSocketIpcServerV1() {

}


void AbulEduLocalSocketIpcServerV1::abeSlotSocketNewConnection() {
    if(m_localDebug)
        qDebug() << "AbulEduLocalSocketIpcServerV1::abeSlotSocketNewConnection (1)";

    QLocalSocket *clientConnection = m_server->nextPendingConnection();

    while (clientConnection->bytesAvailable() < (int)sizeof(quint32))
        clientConnection->waitForReadyRead();


    connect(clientConnection, SIGNAL(disconnected()),
            clientConnection, SLOT(deleteLater()));

    QDataStream in(clientConnection);
    in.setVersion(QDataStream::Qt_4_0);
    if (clientConnection->bytesAvailable() < (int)sizeof(quint16)) {
        return;
    }
    QString message;
    in >> message;

    if(m_useCrypt) {
        SimpleCrypt crypted_message(Q_UINT64_C(0x0c011bfab2fba34e56faeb8a5e4f1ae23e2f0eb4));
        emit abeSignalMessageReceived(crypted_message.decryptToString(message));
    }
    else {
        emit abeSignalMessageReceived(message);
    }
    if(m_localDebug)
        qDebug() << "AbulEduLocalSocketIpcServerV1::abeSlotSocketNewConnection (2)";
}
