/** Classe AbulEduNetworkAccessManagerV1
  * @author 2011-2012 Eric Seigne <eric.seigne@ryxeo.com>
  * @see The GNU Public License (GNU/GPL) v3
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#ifndef ABULEDUNETWORKACCESSMANAGERV1_H
#define ABULEDUNETWORKACCESSMANAGERV1_H

#include <QtNetwork/QNetworkAccessManager>
#include <QtNetwork/QNetworkInterface>
#include <QtNetwork/QNetworkRequest>
#include <QtNetwork/QNetworkReply>
#include <QNetworkAccessManager>
#include <QNetworkProxy>
#include <QDebug>
#include <QRegExp>
#include <QStringList>
#include <QUrl>
#include <QHostInfo>
#include <QSharedPointer>
#include <QNetworkRequest>
#include <QEventLoop>
#include <QNetworkReply>
#include <QObject>
#include <QSettings>

#include "abuledunetworkthreadv1.h"
#include "abuleduproxypassworddialog.h"
#include "abuledulocalsocketipcserverv1.h"
#include "abuledulocalsocketipcclientv1.h"

class AbulEduNetworkAccessManagerV1 : public QNetworkAccessManager
{
    Q_OBJECT

public:
    AbulEduNetworkAccessManagerV1( QObject *parent = 0 );

    /** Permet d'affecter le login/pass pour l'authentification reseau
      * Attention, dès qu'on affecte ces valeurs on accroche le signal / slot pour que cet objet
      * gère tout seul l'authentification
      */
    void setAbeLoginPass(QString login, QString pass);

    /** Permet d'ouvrir la session SSO */
    void abeSSOLogin();

    /** Permet de fermer la session SSO */
    void abeSSOLogout();

    /** Permet de savoir si l'authentification SSO est ok */
    int abeSSOAuthenticationStatus();

    QString abeGetSSOLogin() {return m_login;}
    QString abeGetSSOPassword() {return m_pass;}

public slots:
    /** Gestion de l'erreur "authentification du proxy" */
    void slotProxyAuthenticationRequired(QNetworkProxy proxy,QAuthenticator* authenticator);

    /** Gestion de l'erreur de telechargement */
    void slotReplyError(QNetworkReply::NetworkError code);

    /** Telechargement et configuration du fichier de configuration proxy automatique */
    void finishedSlotWPAD(QNetworkReply* reply);

    /** Permet d'activer ou d'interdire les popup SSO (actif par défaut)*/
    void abeSSOPopupEnable(bool enable);

signals:
    /** Permet de connecter un slot si l'authentification SSO n'est pas bonne */
    void ssoAuthStatus(int ssoAuthErrorValue);

private slots:
    /** Gère l'authentification SSO */
    void slotSSOAuth(QNetworkReply* networkReply,QAuthenticator* authenticator);
    void slotFinishedSSOLogin(QNetworkReply* reply);
    void slotFinishedSSOLogout(QNetworkReply* reply);
    void slotSSOPopupClosed();
    void slotLocalSocketMessageReceived(QString message);
    /** Tentative de gestion des 'erreurs' ssl pour insérer nos certificats -> solution dans le constructeur
      * du manager on ajoute les certificats de manière globale
      */
    void slotSslErrors ( QNetworkReply * reply, const QList<QSslError> & errors );

private:
    bool m_localDebug;
    int                                m_proxyPassAsked;   //Le nombre de fois ou on a demande le mot de passe proxy
    QNetworkReply*                     m_networkReply;     //Le reply qui permet de relancer la connexion sur
                                                           //la même page en cas de capture proxy par ex
    int                                m_ssoAuthentifiedStatus;//un flag pour savoir si oui ou non on est authentifie: 0 on ne sait pas, 1 ok, -1 erreur
    int                                m_passAsked;        //Le nombre de fois ou on a demande le mot de passe sso
    bool    m_ssoPopupIsShow;
    bool    m_ssoUsePopup;     //permet de désactiver totalement les popup, par ex. dans alacarte on gere le sso dans un widget intégré
    QString m_login;
    QString m_pass;
    QString *m_proxyLogin;
    QString *m_proxyPass;

    AbulEduLocalSocketIpcServerV1    *m_localSocketServer; //Un serveur pour passer le ticket SSO aux clients
    AbulEduLocalSocketIpcClientV1    *m_localSocketClient; //Un client pour passer le ticket SSO aux clients

};

#endif // ABULEDUNETWORKACCESSMANAGERV1_H
