/** Classe AbulEduSSOLoginV1 : gestion de l'authentification SSO AbulEdu
  * @author 2012 Eric Seigne <eric.seigne@ryxeo.com>
  * @see The GNU Public License (GNU/GPL) v3
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#include "abuledussologinv1.h"
#include "ui_abuledussologinv1.h"

AbulEduSSOLoginV1::AbulEduSSOLoginV1(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::AbulEduSSOLoginV1)
{
    ui->setupUi(this);
    //A passer a true si vous voulez les infos de debug de ce module
    m_localDebug = false;

    // Si on n'a pas de parent c'est qu'on est une popup, de ce fait on se comporte comme un splashscreen
    // le maitre des arts graphiques nous ayant demandés de faire un truc dont il aurait pas trop honte ...
    if(parent == 0) {
        if(m_localDebug) {
            qDebug() << "AbulEduSSOLoginV1::AbulEduSSOLoginV1 sans parent: on ouvre une popup ...";
        }
        setWindowFlags(Qt::FramelessWindowHint|Qt::WindowStaysOnTopHint|Qt::SplashScreen);
        m_pixmap = QPixmap(":/abuledussologinv1/cadre-identification");
        setAttribute(Qt::WA_TranslucentBackground);
        setFixedSize(m_pixmap.size());

        //On centre le splashscreen
        QRect frect = frameGeometry();
        frect.moveCenter(QDesktopWidget().availableGeometry().center());
        move(frect.topLeft());

        //Evite d'avoir le fond noir quand on est sous linux sans compiz
        this->setMask(m_pixmap.mask());
        ui->frameLogin->setStyleSheet("QObject { \nbackground: transparent; \n}");

        //Si on s'ouvre comme une fenetre -> centré et modal
        move(QApplication::desktop()->screen()->rect().center() - rect().center());
    }
    else {
        if(m_localDebug) {
            qDebug() << "AbulEduSSOLoginV1::AbulEduSSOLoginV1 avec parent: on s'integre dans le paysage ...";
        }
        //Dans le cas contraire c'est qu'on est embarqué en tant que widget, il faut donc s'assurer que le bouton
        //close n'est pas visible
        ui->btnClose->hide();
    }

    // En attentant d'avoir des machines a état partout, on fait comme papy et on essaye de gérer les quelques
    // malheureux composants et leur comportement à la main ... moins de 10, on devrait y arriver !
    ui->btnLogout->hide();
    ui->frmLoginMessage->hide();
    ui->frmOuvrirCompte->hide();

    // On récupère le network access manager
    m_nam = abeApp->getAbeNetworkAccessManager();

    // Un flag pour savoir si l'action de login ou logout est en cours
    m_logoutInProgress = false;

    // Les URL magiques pour changer le mot de passe
    m_rpcWebadminResetPass = new MaiaXmlRpcClient(QUrl("https://webadminxml.abuledu.net/resetpassword.php"), this);

    // Es-ce qu'on propose de s'authentifier sous le nom qu'on a entré la dernière fois ?
    // après avoir demandé à l'équipe la réponse est "oui", on propose même les 5 dernières identités !
    QSettings ssoLoginSettings("abuledu","ssologin");
    QStringList loginDejaUtilises(ssoLoginSettings.value("LastLogins").toStringList());
    QCompleter *loginCompleter = new QCompleter(loginDejaUtilises, this);
    loginCompleter->setCaseSensitivity(Qt::CaseInsensitive);
    ui->leLogin->setCompleter(loginCompleter);
    if(m_localDebug) {
        qDebug() << "AbulEduSSOLoginV1::AbulEduSSOLoginV1 (2)";
    }
}

AbulEduSSOLoginV1::~AbulEduSSOLoginV1()
{
    delete ui;
}

void AbulEduSSOLoginV1::rpcNewPasswordRequested(QVariant &v, QNetworkReply *reply)
{
    if(m_localDebug) {
        qDebug() << "AbulEduSSOLoginV1::rpcNewPasswordRequested";
        qDebug() << "============================================================";
        qDebug() << trUtf8("Détails de la réponse newPassword:");
        qDebug() << v.toMap();
        qDebug() << "============================================================";
    }
    ui->lblLoginMessage->setText(v.toMap().value("Message").toString());
    //Si le message de retour commance par 2 c'est que tout va bien
    if(v.toMap().value("status").toString().startsWith("20")) {
        ui->lblLoginPicto->setPixmap(QPixmap(":/abuledussologinv1/ok"));
    }
    else {
        ui->lblLoginPicto->setPixmap(QPixmap(":/abuledussologinv1/erreur"));
    }
    ui->frmLoginMessage->show();
}

void AbulEduSSOLoginV1::rpcNewAccountRequested(QVariant &v, QNetworkReply *reply)
{
    if(m_localDebug) {
        qDebug() << "AbulEduSSOLoginV1::rpcNewAccountRequested (1)";
        qDebug() << "============================================================";
        qDebug() << trUtf8("Détails de la réponse newPassword:");
        qDebug() << v.toMap();
        qDebug() << "============================================================";
    }
    ui->lblLoginMessage->setText(v.toMap().value("Message").toString());
    //Si le message de retour commance par 2 c'est que tout va bien
    if(v.toMap().value("status").toString().startsWith("20")) {
        ui->lblLoginPicto->setPixmap(QPixmap(":/abuledussologinv1/ok"));
    }
    else {
        ui->lblLoginPicto->setPixmap(QPixmap(":/abuledussologinv1/erreur"));
    }
    ui->frmLoginMessage->show();
    ui->leLogin->setText(ui->leMailOuvrirCompte->text());
    ui->lePassword->setText("");
}

void AbulEduSSOLoginV1::rpcFault(int code, const QString &errorMessage)
{
    if(m_localDebug) {
        qDebug() << "AbulEduSSOLoginV1::rpcFault (1) : " << code << " -> " << errorMessage;
    }

    //Tentative de réouverture d'une session SSO en cas de timeout
    if(code == -32300) {
        m_nam->abeSSOLogin();
    }

}

void AbulEduSSOLoginV1::slotSSOAuthStatus(int code)
{
    if(m_localDebug) {
        qDebug() << "AbulEduSSOLoginV1::slotSSOAuthStatus (1)" << " code " << code;
    }
    QApplication::restoreOverrideCursor();
    disconnect(m_nam, SIGNAL(ssoAuthStatus(int)), this, SLOT(slotSSOAuthStatus(int)));
    if(code == 1) {
        //Tout est OK
        ui->btnLogin->setText("Changer d'utilisateur");
        ui->btnLogin->setIcon(QIcon(":/abuledussologinv1/changer"));
        ui->btnLogin->setEnabled(false);
        ui->btnLogout->show();
        ui->lePassword->setText("");
        ui->lblLoginMessage->setText(trUtf8("Connexion réussie pour l'utilisateur %1.").arg(ui->leLogin->text()));
        ui->lblLoginPicto->setPixmap(QPixmap(":/abuledussologinv1/ok"));
        ui->frmLoginMessage->show();

        //Et on sauvegarde le login dans la liste
        QSettings ssoLoginSettings("abuledu","ssologin");
        QStringList loginDejaUtilises(ssoLoginSettings.value("LastLogins").toStringList());
        //Si on en a déjà 5 on supprime le 1er (le plus ancien) et on enquille les autres
        if(loginDejaUtilises.count() > 4)
            loginDejaUtilises.takeFirst();
        if(!loginDejaUtilises.contains(ui->leLogin->text()))
            loginDejaUtilises << ui->leLogin->text();
        ssoLoginSettings.setValue("LastLogins", loginDejaUtilises);
    }
    else {
        //Si on est en phase de logout ...
        if(m_logoutInProgress) {
            //Ou si erreur
            ui->lblLoginMessage->setText(trUtf8("Session terminée. Vous pouvez maintenant vous authentifier sous une autre identité."));
            ui->lblLoginPicto->setPixmap(QPixmap(":/abuledussologinv1/ok"));

            m_logoutInProgress = false;
        }
        else {
            //Ou si erreur
            ui->lblLoginMessage->setText(trUtf8("Identifiant ou mot de passe incorrect ..."));
            ui->lblLoginPicto->setPixmap(QPixmap(":/abuledussologinv1/erreur"));

        }
        ui->frmLoginMessage->show();
        ui->btnLogin->setEnabled(true);
        ui->btnLogout->hide();
    }
    emit signalSSOAuthDone(code);
    if(m_localDebug) {
        qDebug() << "AbulEduSSOLoginV1::slotSSOAuthStatus (2)";
    }
}

void AbulEduSSOLoginV1::on_btnLogin_clicked()
{
    if(m_localDebug) {
        qDebug() << "AbulEduSSOLoginV1::on_btnLogin_clicked";
    }
    if(ui->leLogin->text().trimmed() != "" && ui->lePassword->text().trimmed() != "") {
        ui->frmLoginMessage->hide();
        ui->frmOuvrirCompte->hide();

        if(m_nam->abeSSOAuthenticationStatus() == 1) {
            if(m_nam->abeGetSSOLogin() == ui->leLogin->text() &&
                    m_nam->abeGetSSOPassword() == ui->lePassword->text()) {
                ui->lblLoginMessage->setText(trUtf8("Vous êtes déjà authentifié(e) avec ce compte utilisateur. Si vous voulez vous authentifier avec un autre compte, changez les valeurs dans les champs identifiants et mot de passe."));
                ui->lblLoginPicto->setPixmap(QPixmap(":/abuledussologinv1/attention"));

                ui->frmLoginMessage->show();
            }
        }
        else {
            QApplication::setOverrideCursor(Qt::WaitCursor);
            //On passe en prod sur la boutique en ligne ...
            m_webadminPassAsked = 0;
            //1. Authentification
            m_nam->setAbeLoginPass(ui->leLogin->text(), ui->lePassword->text());
            //On deconnecte une eventuelle connexion precedente
            disconnect(m_nam, SIGNAL(ssoAuthStatus(int)), this, SLOT(slotSSOAuthStatus(int)));
            connect(m_nam, SIGNAL(ssoAuthStatus(int)), this, SLOT(slotSSOAuthStatus(int)));
            m_nam->abeSSOLogin();
        }
    }
    else {
        ui->frmLoginMessage->show();
        ui->lblLoginPicto->setPixmap(QPixmap(":/abuledussologinv1/attention"));
        ui->lblLoginMessage->setText(trUtf8("Veuillez entrer un identifiant et un mot de passe ..."));
        ui->leLogin->setFocus();
    }
}

void AbulEduSSOLoginV1::on_btnPasswordOublie_clicked()
{
    if(m_localDebug) {
        qDebug() << "AbulEduSSOLoginV1::on_btnPasswordOublie_clicked";
    }
    if(ui->leLogin->text().trimmed() == "") {
        ui->frmLoginMessage->show();
        ui->lblLoginPicto->setPixmap(QPixmap(":/abuledussologinv1/attention"));
        ui->lblLoginMessage->setText(trUtf8("Veuillez entrer une adresse email valide ..."));
        ui->leLogin->setFocus();
    }
    else {
        ui->frmLoginMessage->hide();
        ui->frmOuvrirCompte->hide();

        //Mot de passe oublié, merci de m'en envoyer un nouveau ... requete a faire vers le webadmin/portail
        //On peut essayer de se lancer direct sur le XMLRPC
        QList<QVariant> params;
        params << ui->leLogin->text().trimmed();
        m_rpcWebadminResetPass->call("abuledu.resetpassword.sendNewPassword", params,
                                     this, SLOT(rpcNewPasswordRequested(QVariant&, QNetworkReply*)),
                                     this, SLOT(rpcFault(int, const QString &)));
    }
}

void AbulEduSSOLoginV1::on_btnOuvrirUnCompte_clicked()
{
    if(m_localDebug) {
        qDebug() << "AbulEduSSOLoginV1::on_btnOuvrirUnCompte_clicked";
    }
    //Formulaire d'ouverture de compte ... sur la boutique ?
    //on affiche un mini formulaire de demande d'adresse email
    ui->frmLoginMessage->hide();
    ui->frmOuvrirCompte->show();
}

void AbulEduSSOLoginV1::paintEvent(QPaintEvent *pe)
{
    if(m_localDebug) {
        qDebug() << "AbulEduSSOLoginV1::paintEvent";
    }
    QPainter aPainter(this);
    aPainter.drawPixmap(rect(), m_pixmap);
}

void AbulEduSSOLoginV1::on_btnLogout_clicked()
{
    if(m_localDebug) {
        qDebug() << "AbulEduSSOLoginV1::on_btnLogout_clicked";
    }
    m_logoutInProgress = true;
    ui->frmOuvrirCompte->hide();
    ui->frmLoginMessage->hide();
    disconnect(m_nam, SIGNAL(ssoAuthStatus(int)), this, SLOT(slotSSOAuthStatus(int)));
    connect(m_nam, SIGNAL(ssoAuthStatus(int)), this, SLOT(slotSSOAuthStatus(int)));
    m_nam->abeSSOLogout();
}

void AbulEduSSOLoginV1::on_btnCreerUnCompte_clicked()
{
    if(m_localDebug) {
        qDebug() << "AbulEduSSOLoginV1::on_btnCreerUnCompte_clicked";
    }
    if(ui->leMailOuvrirCompte->text().trimmed() == "") {
        ui->frmLoginMessage->show();
        ui->lblLoginPicto->setPixmap(QPixmap(":/abuledussologinv1/attention"));
        ui->lblLoginMessage->setText(trUtf8("Veuillez entrer une adresse email valide ..."));
        ui->leMailOuvrirCompte->setFocus();
    }
    else {
        QList<QVariant> params;
        params << ui->leMailOuvrirCompte->text().trimmed();
        m_rpcWebadminResetPass->call("abuledu.resetpassword.createNewAccount", params,
                                     this, SLOT(rpcNewAccountRequested(QVariant&, QNetworkReply*)),
                                     this, SLOT(rpcFault(int, const QString &)));
    }
    //1. côté serveur -> vérifie que l'email est ok
    //1.bis le serveur retourne un mot de passe dans son XMLRPC (comme ça l'utilisateur peut bosser tout de suite)
    //1.ter envoie un mail avec une adresse pour confirmer l'ouverture du compte
    //2. lorsque l'utilisateur confirme l'ouverture du compte -> ouvre le compte sur la boutique
    //2.bis et demande prénom + nom
    //2.ter affiche offre spéciale
}

void AbulEduSSOLoginV1::showEvent(QShowEvent *event)
{
    if(m_localDebug) {
        qDebug() << "AbulEduSSOLoginV1::showEvent";
    }
    if(m_nam->abeSSOAuthenticationStatus() == 1) {
        ui->btnLogin->setText("Changer d'utilisateur");
        ui->btnLogin->setIcon(QIcon(":/abuledussologinv1/changer"));
        ui->btnLogin->setEnabled(false);
        ui->btnLogout->show();
    }
    else {
        ui->btnLogin->setEnabled(true);
        ui->btnLogout->hide();
    }
}

void AbulEduSSOLoginV1::on_btnClose_clicked()
{
    if(m_localDebug) {
        qDebug() << "AbulEduSSOLoginV1::on_btnClose_clicked";
    }
    emit signalClosed();
    close();
}
